/**********
This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the
Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version. (See <http://www.gnu.org/copyleft/lesser.html>.)

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
more details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**********/
// "liveMedia"
// Author Flavio Oliveira
// Derived from MP3 File Sources
// Implementation

#include "AACAudioFileSource.hh"
#include "AACInternals.hh"
#include "InputFile.hh"

#define BUFSIZE 8192

#ifdef BSD
static struct timezone Idunno;
#else
static int Idunno;
#endif

FILE* fid;
////////// AACFileSource //////////

AACFileSource::AACFileSource(UsageEnvironment& env, FILE* fid)
  : FramedFileSource(env, fid), nData(0), offset(0), buffer(new char[BUFSIZE]),
    fStreamState(new AACFrame()) {
}

AACFileSource::~AACFileSource() {
  delete fStreamState;
}

char const* AACFileSource::MIMEtype() const {
  return "audio/MPEG";
}

AACFileSource* AACFileSource::createNew(UsageEnvironment& env, char const* fileName) {
    
  AACFileSource* newSource = NULL;

  do {
    
    fid = OpenInputFile(env, fileName);
    if (fid == NULL) break;

    newSource = new AACFileSource(env, fid);
    if (newSource == NULL) break;

    unsigned fileSize = GetFileSize(fileName, fid);
    newSource->assignStream(fid, fileSize);
    if (!newSource->initializeStream()) break;

    return newSource;
  } while (0);

  delete newSource;
  return NULL;
}

void AACFileSource::doGetNextFrame() {
  if (!doGetNextFrame1()) {
    handleClosure(this);
    return;
  }

  // Switch to another task:
#if defined(__WIN32__) || defined(_WIN32)
  // HACK: liveCaster/lc uses an implementation of scheduleDelayedTask()
  // that performs very badly (chewing up lots of CPU time, apparently polling)
  // on Windows.  Until this is fixed, we just call our "afterGetting()"
  // function directly.  This avoids infinite recursion, as long as our sink
  // is discontinuous, which is the case for the RTP sink that liveCaster/lc
  // uses. #####
  afterGetting(this);
#else
  nextTask() = envir().taskScheduler().scheduleDelayedTask(0,
				(TaskFunc*)afterGetting, this);
#endif
}

Boolean AACFileSource::doGetNextFrame1() {
  readFile();
  findADTSHeader();
  getNextFrame(fTo,fFrameSize);
        	
  // Set the 'presentation time':
  if (fPresentationTime.tv_sec == 0 && fPresentationTime.tv_usec == 0) {
     // This is the first frame, so use the current time:
     gettimeofday(&fPresentationTime, &Idunno);
     fPresentationTime.tv_usec = 1;
     fPresentationTime.tv_sec = 0;
  } else {
     // Increment by the play time of the previous frame (1024*/iSrate Seconds)
     unsigned uSeconds = (int)(fPresentationTime.tv_usec + 1024./(float)iSrate*1000000.);
     fPresentationTime.tv_sec += uSeconds/1000000; //bf
     fPresentationTime.tv_usec = uSeconds%1000000; //bf
  }
  fDurationInMicroseconds = 1024*1000000/iSrate; //1024./(float)iSrate*1000000.;
   
  if ( (nread == 0) && (nData == 0) ) return False;
  else return True;
}

void AACFileSource::assignStream(FILE* fid, unsigned fileSize) {
  n = nread = 0;
  frameSize = 0;  
}

Boolean AACFileSource::initializeStream() {
  fHaveJustInitialized = True;
  return True; 		
}

Boolean AACFileSource::findADTSHeader() {
  res = fStreamState->findADTSHeader((const unsigned char*)buffer + offset, nData - offset, nread, &fHDR);
  setSampleFrequency();		    
	
  if (!res) {
      return False;
  }
	
  if (nData - offset < fHDR.aac_frame_length)  {
  	res=False;
	nread = 0 ;
  }

  frameSize = fHDR.aac_frame_length - fHDR.header_size;
  headerSize = fHDR.header_size;
    
  return True;
}

Boolean AACFileSource::findADIFHeader() {
  return fStreamState->isADIFHeader((const unsigned char*)buffer + offset, nData - offset, nread);
}

void AACFileSource::getNextFrame(unsigned char* outBuf, unsigned& resultFrameSize) {
  if (res == true) {
  	  offset += nread ; // skip header
      //memmove(outBuf, buffer+offset+1, frameSize-1); // without ADTS header	      
      memmove(outBuf, buffer+offset-headerSize, frameSize+headerSize); // with ADTS Header	      
	
	  //resultFrameSize = frameSize-1; // without ADTS header
	  resultFrameSize = frameSize+headerSize; // with ADTS Header
	
  	  offset += frameSize ; // skip rest of frame too 
  	  nread = resultFrameSize+1; 
  }
}	

void AACFileSource::readFile() {
  if (res == False) {
	  nData -= offset ;
	  memmove(buffer, buffer+offset, nData) ;
   	  n = fread(buffer+nData,1,BUFSIZE-nData,fid);
	  nData += n ;
	  offset = 0;
  }	
}

void AACFileSource::setSampleFrequency() {
  if (iSrate!= 0)
     return;
       
  switch(fHDR.sample_frequency_index) {
 	 case 0: 
		iSrate  =  96000;
		break;
	 case 1: 
		iSrate  =  88200;
		break;
	 case 2: 
		iSrate  =  64000;
		break;
	 case 3: 
		iSrate  =  48000;
		break;
	 case 4: 
		iSrate  =  44100;
		break;
	 case 5: 
		iSrate  =  32000;
		break;
	 case 6: 
		iSrate  =  24000;
		break;
	 case 7: 
		iSrate  =  22050;
		break;
	 case 8: 
		iSrate  =  16000;
		break;
	 case 9: 
		iSrate  =  12000;
		break;
	 case 10: 
		iSrate  =  11025;
		break;
	 case 11: 
		iSrate  =  8000;
		break;
	 default: 
		iSrate  =  44100;
		break;
  }	
}	
