/**********
This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the
Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version. (See <http://www.gnu.org/copyleft/lesser.html>.)

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
more details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**********/

// Copyright (c) 1996-2000, Live Networks, Inc.  All rights reserved
// A test program that receives a RTP/RTCP multicast MP3 stream,
// and outputs the resulting MP3 file stream to 'stdout'
// main program

#include "liveMedia.hh"
#include "GroupsockHelper.hh"
#include "BasicUsageEnvironment.hh"

// A structure to hold the state of the current session.
// It is used in the "afterPlaying()" function to clean up the session.
struct sessionState_t {
  FramedSource* source;
  FileSink* sink;
  RTCPInstance* rtcpInstance;
} sessionState;

void saveHeader();
void addADIFHeader(unsigned short avgBitRate, unsigned short samplingFrequency, unsigned short mode, unsigned short channelConfig);
void afterPlaying(void* clientData); // forward

struct adifHeader {
   unsigned short avgBitRate;
   unsigned short samplingFrequency;
   unsigned short mode;
   unsigned short channelConfig;
} adif;
   
UsageEnvironment* env;

int main(int argc, char** argv) {
	
  saveHeader();
  
  // Begin by setting up our usage environment:
  TaskScheduler* scheduler = BasicTaskScheduler::createNew();
  env = BasicUsageEnvironment::createNew(*scheduler);

  // Create the data sink for 'stdout':
  sessionState.sink = FileSink::createNew(*env, "stdout");
  // Note: The string "stdout" is handled as a special case.
  // A real file name could have been used instead.

  // Create 'groupsocks' for RTP and RTCP:
  char* sessionAddressStr
#ifdef USE_SSM
    = "232.255.42.42";
#else
    = "239.255.42.42";
  // Note: If the session is unicast rather than multicast,
  // then replace this string with "0.0.0.0"
#endif
  const unsigned short rtpPortNum = 15960;
  const unsigned short rtcpPortNum = rtpPortNum+1;
#ifndef USE_SSM
  const unsigned char ttl = 1; // low, in case routers don't admin scope
#endif
  
  struct in_addr sessionAddress;
  sessionAddress.s_addr = our_inet_addr(sessionAddressStr);
  const Port rtpPort(rtpPortNum);
  const Port rtcpPort(rtcpPortNum);
  
#ifdef USE_SSM
  char* sourceAddressStr = "aaa.bbb.ccc.ddd";
                           // replace this with the real source address
  struct in_addr sourceFilterAddress;
  sourceFilterAddress.s_addr = our_inet_addr(sourceAddressStr);

  Groupsock rtpGroupsock(*env, sessionAddress, sourceFilterAddress, rtpPort);
  Groupsock rtcpGroupsock(*env, sessionAddress, sourceFilterAddress, rtcpPort);
  rtcpGroupsock.changeDestinationParameters(sourceFilterAddress,0,~0);
      // our RTCP "RR"s are sent back using unicast
#else
  Groupsock rtpGroupsock(*env, sessionAddress, rtpPort, ttl);
  Groupsock rtcpGroupsock(*env, sessionAddress, rtcpPort, ttl);
#endif
  
  RTPSource* rtpSource;
  unsigned char rtpPayloadFormat = 96; // a dynamic payload type
  rtpSource
    = MPEG4GenericRTPSource::createNew(*env, &rtpGroupsock, rtpPayloadFormat, 48000,
    									"audio", "aac-hbr", 13,3,3);

  // Create (and start) a 'RTCP instance' for the RTP source:
  const unsigned estimatedSessionBandwidth = 160; // in kbps; for RTCP b/w share
  const unsigned maxCNAMElen = 100;
  unsigned char CNAME[maxCNAMElen+1];
  gethostname((char*)CNAME, maxCNAMElen);
  CNAME[maxCNAMElen] = '\0'; // just in case
  sessionState.rtcpInstance
    = RTCPInstance::createNew(*env, &rtcpGroupsock,
			      estimatedSessionBandwidth, CNAME,
			      NULL /* we're a client */, rtpSource);
  // Finally, start receiving the multicast stream:
  *env << "Beginning receiving multicast stream...\n";
  sessionState.source = rtpSource;
  sessionState.sink->startPlaying(*sessionState.source, afterPlaying, NULL);

  env->taskScheduler().doEventLoop(); // does not return

  return 0; // only to prevent compiler warning
}


void afterPlaying(void* /*clientData*/) {
  *env << "...done receiving\n";

  // End by closing the media:
  Medium::close(sessionState.rtcpInstance); // Note: Sends a RTCP BYE
  Medium::close(sessionState.sink);
  Medium::close(sessionState.source);
}

// m=audio 49230 RTP/AVP 96 a=rtpmap:96 mpeg4-generic/48000/6 a=fmtp:96 
// streamtype=5; profile-level-id=16; mode=AAC-hbr; config=11B0; 
// sizeLength=13; indexLength=3; indexDeltaLength=3; constantDuration=1024
// Specifies a 5.1 channel AAC stream with a sampling rate of 48 kHz.

void saveHeader()
{
   adif.avgBitRate = 128000; // Calc bitrate
   adif.samplingFrequency = 48000;
   adif.mode = 2;
   adif.channelConfig = 1;
   addADIFHeader(adif.avgBitRate, adif.samplingFrequency, adif.mode, adif.channelConfig);
}  

//  <<    bitwise shift left     a << 2  /* a shifted left by two places */
//  >>    bitwise shift right
//  &     bitwise AND            a & 3   /* extracts the two least significant bits */
//  &     address-of
  
void addADIFHeader(unsigned short avgBitRate, unsigned short samplingFrequency, unsigned short mode, unsigned short channelConfig)
{
   unsigned int sf;	
   unsigned char ADIFHeader[] = {
	   0x41, 0x44, 0x49, 0x46, 0x00, 0x2E, 0x00, 0x00, 0x00, 0x00,
	   0x00, 0xA8, 0x80, 0x00, 0x04, 0x00, 0x01, 0x00
   };

  unsigned char temp;
  unsigned short *tmp = reinterpret_cast <unsigned short *> (ADIFHeader);

   tmp[2] = ( avgBitRate & 0x007ff800 ) >> 11;
   tmp[3] = ( avgBitRate & 0x000007ff ) << 5;
   switch(samplingFrequency) {

		case 96000: 
			sf  =  0x00;
			break;
		case 88200: 
			sf  =  0x08;
			break;
		case 64000: 
			sf  =  0x10;
			break;
		case 48000: 
			sf  =  0x18;
			break;
		case 44100: 
			sf  =  0x20;
			break;
		case 32000: 
			sf  =  0x28;
			break;
		case 24000: 
			sf  =  0x30;
			break;
		case 22050: 
			sf  =  0x38;
			break;
		case 16000: 
			sf  =  0x40;
			break;
		case 12000: 
			sf  =  0x48;
			break;
		case 11025: 
			sf  =  0x50;
			break;
		case 8000: 
			sf  =  0x58;
			break;
		default: 
			sf  =  0x20;
			break;
	}
    
    ADIFHeader[8] = (ADIFHeader[8] & 0xFE7F) | ( (unsigned short)( mode - 1 ) << 7 );
    ADIFHeader[9] = (unsigned short) sf;  
    
	//tmp[5] = (tmp[5] & 0xFE7F) | ( (unsigned short)( mode - 1 ) << 7 );
	//tmp[5] = (tmp[5] & 0xFF87) | (unsigned short) sf;  
	
	if (channelConfig == 1)  tmp[7] &= 0x00FF;
	else tmp[7] &= 0x04FF;
	
  FILE* fid;
  fid = fopen("saida.aac", "wb");
  fwrite(tmp, 1, 19, fid);
  fclose(fid);	
}			
